require 'util'

Debug = {}

Debug.enabled = true

function Debug:onMapCreated()
  local size = 500

  local surface = game.surfaces["nauvis"]
  if game.active_mods["The_Lab_tiles"] then
    -- Let's create lab tiles
    local labTiles = {}
    for X = -size/2,size/2 do
      for Y = -size/2,size/2 do
        if (X+Y) %2 == 0 then
          table.insert(labTiles, {name = "lab-dark-1", position = {X,Y} })
        else
          table.insert(labTiles, {name = "lab-dark-2", position = {X,Y} })
        end
      end
    end
    surface.set_tiles(labTiles)

  else
    -- If no lab tiles, just delete the water
    local waterTiles = {}
    for _, waterTile in pairs(surface.find_tiles_filtered{
      area =
      {
        left_top = {-size/2, -size/2},
        right_bottom = {size/2, size/2},
      },
      collision_mask = "water-tile"
    }) do
      if waterTile.valid then
        table.insert(waterTiles, {name = "grass-1", position = util.table.deepcopy(waterTile.position)})
      end
    end
    surface.set_tiles(waterTiles)
  end

  self:createTestbench(surface)
end



function Debug:onPlayerCreated(player_index)
  -- research all technologies
  game.players[player_index].force.research_all_technologies()

  local player = game.players[player_index]

    player.insert("solar-panel")
    player.insert("accumulator")
    player.insert("substation")

    player.insert("rail")
    player.insert("rail-signal")
    player.insert("train-stop")
    player.insert("trainassembly")
    player.insert("traincontroller")
    player.insert("traindepot")

    player.insert("locomotive")
    player.insert("trainassembly-recipefuel")
end



function Debug:createTestbench(surface)
  local areaRange = 60

  -- STEP 1: delete all entities surrounding the test surface
  surface.destroy_decoratives{area = {{-areaRange, -areaRange}, {areaRange, areaRange}}}
  areaRange = math.floor(.5 + areaRange / 2)
  for _, entity in pairs(surface.find_entities{{-areaRange, -areaRange}, {areaRange, areaRange}}) do
    entity.destroy()
  end

  if true then return end -- waiting on v0.17.42 as events are not raised in on_init

  -- STEP 2: Create the test bench
  -- STEP 2a:create the rails
  local distance = math.floor(.5 + areaRange/2)
  local areaMax = math.floor(.5 + (areaRange*.8)/2)*2
  for i = 2, areaMax, 2 do
    surface.create_entity{
      name        = "straight-rail",
      position    = {x = i, y = distance},
      direction   = defines.direction.east,
      force       = "player",
    }
    surface.create_entity{
      name        = "straight-rail",
      position    = {x = -i, y = distance},
      direction   = defines.direction.west,
      force       = "player",
    }
    surface.create_entity{
      name        = "straight-rail",
      position    = {x = distance, y = -i},
      direction   = defines.direction.north,
      force       = "player",
    }
    surface.create_entity{
      name        = "straight-rail",
      position    = {x = -distance, y = -i},
      direction   = defines.direction.south,
      force       = "player",
    }
  end

  -- Step 2b:create the train depots
  surface.create_entity{
    name        = "traindepot",
    position    = {x = 2, y = distance-2},
    direction   = defines.direction.west,
    force       = "player",
    raise_built = true,
  }
  surface.create_entity{
    name        = "traindepot",
    position    = {x = -2, y = distance+2},
    direction   = defines.direction.east,
    force       = "player",
    raise_built = true,
  }
  surface.create_entity{
    name        = "traindepot",
    position    = {x = distance-2, y = -2},
    direction   = defines.direction.south,
    force       = "player",
    raise_built = true,
  }
  surface.create_entity{
    name        = "traindepot",
    position    = {x = -distance+2, y = -areaMax},
    direction   = defines.direction.north,
    force       = "player",
    raise_built = true,
  }

  -- Step 2c:create the train assemblers
  -- TODO

  -- Step 2d:create the traincontrollers
  -- TODO
end
